# !/usr/bin/python
# coding=utf-8
import sys


__package__ = 'mayatk'
__version__ = '0.6.1'


def __getattr__(attr):
	"""This function dynamically imports a module and returns an attribute from the module. 

	Parameters:
		attr (str): The name of the attribute to be imported. The name should be in the format 
					'module_name.attribute_name' or just 'attribute_name'.
	Return:
		(obj) The attribute specified by the `attr` argument.

	:Raises:
		AttributeError: If the specified attribute is not found in either the original module 
						or the 'Core' module within the package.
	Example:
		<package>.__getattr__('module1.attribute1') #returns: <attribute1 value>
		<package>.__getattr__('attribute1') #returns: <attribute1 value>
	"""
	try:
		module = __import__(f"{__package__}.{attr}", fromlist=[f"{attr}"])
		setattr(sys.modules[__name__], attr, getattr(module, attr))
		return getattr(module, attr)

	except (ValueError, ModuleNotFoundError):
		module = __import__(f"{__package__}.Core", fromlist=["Core"])
		return getattr(module, attr)

	except AttributeError as error:
		raise AttributeError(f"Module '{__package__}' has no attribute '{attr}'") from error

# --------------------------------------------------------------------------------------------









# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# --------------------------------------------------------------------------------------------
# deprecated:
# --------------------------------------------------------------------------------------------


# !/usr/bin/python
# coding=utf-8
import os, sys
import unittest
import inspect

import pymel.core as pm

from mayatk.Core import Core


# sfr = pm.melGlobals['cmdScrollFieldReporter']
# pm.cmdScrollFieldReporter(sfr, edit=1, clear=1)


class Main(unittest.TestCase):
	'''
	'''
	@staticmethod
	def replace_mem_address(obj):
		'''Replace memory addresses in a string representation of an object with a fixed format of '0x00000000000'.

		Parameters:
			obj (object): The input object. The function first converts this object to a string using the `str` function.

		Return:
			(str) The string representation of the object with all memory addresses replaced.

		Example:
			>>> replace_mem_address("<class 'str'> <PySide2.QtWidgets.QWidget(0x1ebe2677e80, name='MayaWindow') at 0x000001EBE6D48500>")
			"<class 'str'> <PySide2.QtWidgets.QWidget(0x00000000000, name='MayaWindow') at 0x00000000000>"
		'''
		import re
		return re.sub(r'0x[a-fA-F\d]+', '0x00000000000', str(obj))

	def perform_test(self, case):
		'''
		'''
		for expression, expected_result in case.items():
			m = str(expression).split('(')[0] #ie. 'self.setCase' from "self.setCase('xxx', 'upper')"

			try:
				path = os.path.abspath(inspect.getfile(eval(m)))
			except TypeError as error:
				path = ''

			result = eval(expression)
			self.assertEqual(
				result, 
				expected_result, 
				f"\n\n# Error: {path}\n#\tCall: {expression.replace('self.', '', 1)}\n#\tExpected {type(expected_result)}: {expected_result}\n#\tReturned {type(result)}: {result}"
			)



class Core_test(Main, Core):
	'''
	set object mode:
		pm.selectMode(object=1)

	set component mode:
		pm.selectMode(component=1)

	set component mode type:
		pm.selectType(allObjects=1)
		pm.selectType(mc=1)
		pm.selectType(vertex=1)
		pm.selectType(edge=1)
		pm.selectType(facet=1)
		pm.selectType(polymeshUV=1)
		pm.selectType(meshUVShell=1)
	'''
	#test imports:
	import mayatk as mtk
	from mayatk import Cmpt
	from mayatk.Cmpt import getComponents

	#Tear down the any previous test by creating a new scene:
	pm.mel.file(new=True, force=True)

	#assemble the test scene:
	if not pm.objExists('cyl'):
		cyl = pm.polyCylinder(radius=5, height=10, subdivisionsX=12, subdivisionsY=1, subdivisionsZ=1, name='cyl')

	def test_undo(self):
		'''
		'''
		self.perform_test({
			"self.replace_mem_address(self.undo())": '<function Core.undo.<locals>.wrapper at 0x00000000000>',
		})

	def test_getMainWindow(self):
		'''
		'''
		self.perform_test({
			"self.replace_mem_address(self.getMainWindow())": "<PySide2.QtWidgets.QWidget(0x00000000000, name=\"MayaWindow\") at 0x00000000000>" or None,
		})

	def test_mfnMeshGenerator(self):
		'''
		'''
		self.perform_test({
			"str(next(self.mfnMeshGenerator('cyl'))).split(';')[0]": '<maya.OpenMaya.MFnMesh',
		})

	def test_getArrayType(self):
		'''
		'''
		self.perform_test({
			"self.getArrayType(100)": 'int',
			"self.getArrayType('cylShape.vtx[:]')": 'str',
			"self.getArrayType(pm.ls('cylShape.vtx[:]'))": 'obj',
		})

	def test_convertArrayType(self):
		'''
		'''
		self.perform_test({
			"self.convertArrayType('cyl.vtx[:2]', 'str')": ['cylShape.vtx[0:2]'],
			"self.convertArrayType('cyl.vtx[:2]', 'str', flatten=True)": ['cylShape.vtx[0]', 'cylShape.vtx[1]', 'cylShape.vtx[2]'],
			"str(self.convertArrayType('cyl.vtx[:2]', 'obj'))": "[MeshVertex('cylShape.vtx[0:2]')]",
			"str(self.convertArrayType('cyl.vtx[:2]', 'obj', flatten=True))": "[MeshVertex('cylShape.vtx[0]'), MeshVertex('cylShape.vtx[1]'), MeshVertex('cylShape.vtx[2]')]",
			"self.convertArrayType('cyl.vtx[:2]', 'int')": [0, 2],
			"self.convertArrayType('cyl.vtx[:2]', 'int', flatten=True)": [0, 1, 2],
		})

	def test_getParameterValuesMEL(self):
		'''
		'''
		self.perform_test({
			# "self.getParameterValuesMEL()": None,
		})

	def test_setParameterValuesMEL(self):
		'''
		'''
		self.perform_test({
			# "self.setParameterValuesMEL()": None,
		})

	def test_getSelectedChannels(self):
		'''
		'''
		self.perform_test({
			# "self.getSelectedChannels()": None,
		})

	def test_getPanel(self):
		'''
		'''
		self.perform_test({
			# "self.getPanel()": None,
		})

	def test_mainProgressBar(self):
		'''
		'''
		self.perform_test({
			# "self.mainProgressBar()": None,
		})

	def test_viewportMessage(self):
		'''
		'''
		self.perform_test({
			# "self.viewportMessage()": None,
		})

# -----------------------------------------------------------------------------









# -----------------------------------------------------------------------------

if __name__=='__main__':

	unittest.main(exit=False)

# -----------------------------------------------------------------------------
# Notes
# -----------------------------------------------------------------------------

# """

# def test_(self):
# 	'''
# 	'''
# 	self.perform_test({
# 		"self.()": None,
# 	})

# def test_(self):
# 	'''
# 	'''
# 	self.perform_test({
# 		# "self.": '',
# 	})

# def test_(self):
# 	'''
# 	'''
# 	self.perform_test({
# 		# "self.": '',
# 	})

# def test_(self):
# 	'''
# 	'''
# 	self.perform_test({
# 		# "self.": '',
# 	})
# """

# Deprecated ---------------------


# ------------------------------------------------------------------------------------
# this is the missing stuff when running python.exe compared with mayapy.exe

# mayaver = 2022
# pythonver = 37

# mayapath = '%ProgramFiles%/Autodesk/Maya{}'.format(mayaver)

# os.environ['MAYA_LOCATION'] = mayapath
# os.environ['PYTHONHOME'] = mayapath+'/Python{}'.format(mayaver, pythonver)
# os.environ['PATH'] = mayapath+'/bin;'.format(mayaver) + os.environ['PATH']

# from pythontk import File
# for d in [
# 	'{}/bin'.format(mayapath), 
# 	'{}/bin3'.format(mayapath), 
# 	'{}/Python{}'.format(mayapath, pythonver)
# 	]:
# 	for dd in File.getDirContents(d, 'dirpaths', excDirs='Python27',  recursive=True):
# 		print (dd)
# 		sys.path.append(dd)

# import maya.standalone
# maya.standalone.initialize(name='python')


# !/usr/bin/python
# coding=utf-8
import importlib


def run_test_cases(test_module):
	"""Runs all test cases within a test module.

	Parameters:
		test_module (str)(module): A module object or a string representing the module name.

	"""
	if isinstance(test_module, str):
		test_module = importlib.import_module(test_module)
		
	for name, obj in test_module.__dict__.items():
		if isinstance(obj, type) and obj.__module__ == test_module.__name__:
			obj = obj()
			for method_name in dir(obj):
				if method_name.startswith("test_"):
					test_case = getattr(obj, method_name)
					test_case()


def run_tests(module_names):
	"""Reloads the non-test and test modules and runs the test cases within the test modules.

	Parameters:
		module_names (list): A list of strings representing the names of the non-test modules.

	"""
	for module_name in module_names:
		module = __import__(f"mayatk.{module_name}", fromlist=[module_name])
		test_module = __import__(f"test.{module_name}_test", fromlist=[f"{module_name}_test"])
		importlib.reload(module)
		importlib.reload(test_module)
		msg_start = f'-> Starting tests for {module_name} ..'
		print (f"{'-'*len(msg_start)}\n{msg_start}")
		run_test_cases(test_module)
		print (f"<- {module_name} tests completed.\n{'-'*len(msg_start)}")

# --------------------------------------------------------------------------------------------

if __name__=='__main__':

	run_tests([
		'Core',
		'Node',
		'Cmpt',
		'Edit',
		'Xform',
		'Rig',
	])

# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------



# --------------------------------------------------------------------------------------------
# deprecated:
# --------------------------------------------------------------------------------------------

# from mayatk import Core; from test import core_test; importlib.reload(Core); importlib.reload(core_test); run_tests(core_test)
# from mayatk import Node; from test import node_test; importlib.reload(Node); importlib.reload(node_test); run_tests(node_test)
# from mayatk import Cmpt; from test import cmpt_test; importlib.reload(Cmpt); importlib.reload(cmpt_test); run_tests(cmpt_test)
# from mayatk import Edit; from test import edit_test; importlib.reload(Edit); importlib.reload(edit_test); run_tests(edit_test)
# from mayatk import Xform; from test import xform_test; importlib.reload(Xform); importlib.reload(xform_test); run_tests(xform_test)
# from mayatk import Rig; from test import rig_test; importlib.reload(Rig); importlib.reload(rig_test); run_tests(rig_test)

# from test import core_test;  core_test.unittest.main(exit=False)
# from test import edit_test;  edit_test.unittest.main(exit=False)
# from test import cmpt_test;  cmpt_test.unittest.main(exit=False)
# from test import rig_test;   rig_test.unittest.main(exit=False)
# from test import xform_test; xform_test.unittest.main(exit=False)


# !/usr/bin/python
# coding=utf-8
import os, sys

try:
	import pymel.core as pm
except ImportError as error:
	print (f'# Error: {__file__}: {error}')

from pythontk import Iter
from mayatk import Node


class Core():
	'''
	'''
	def undo(fn):
		'''A decorator to place a function into Maya's undo chunk.
		Prevents the undo queue from breaking entirely if an exception is raised within the given function.

		Parameters:
			fn (obj): The decorated python function that will be placed into the undo que as a single entry.
		'''
		def wrapper(*args, **kwargs):
			with pm.UndoChunk():
				rtn = fn(*args, **kwargs)
				return rtn
		return wrapper


	@staticmethod
	def getMainWindow():
		'''Get maya's main window object.

		Return:
			(QWidget)
		'''
		from PySide2.QtWidgets import QApplication

		app = QApplication.instance()
		if not app:
			return print(f'# Warning: {__file__} in getMainWindow\n#\tCould not find QApplication instance.')

		main_window = next(iter(w for w in app.topLevelWidgets() if w.objectName()=='MayaWindow'), None)
		if not main_window:
			return print(f'# Warning: {__file__} in getMainWindow\n#\tCould not find main window instance.')

		return main_window


	@staticmethod
	def appendMayaPaths(maya_version=2023):
		"""Adds the required library paths for Maya to the system's Python path.

		Parameters:
			maya_version (int, str, optional): The version of Maya to add the paths for. Defaults to 2023.

		Return:
			(None)
		"""
		maya_install_path = os.environ.get("MAYA_LOCATION") #Get the Maya installation path.
		if not maya_install_path:
			return print(f"# Error: {__file__} in appendMayaPaths\n#\tCould not find the Maya installation path.\n#\tPlease make sure that the MAYA_LOCATION environment variable is set.")

		#Add the Maya libraries to the Python path.
		os.environ["PYTHONHOME"] = os.path.join(maya_install_path, "Python")
		os.environ["PATH"] = os.path.join(maya_install_path, "bin") + ";" + os.environ["PATH"]

		sys.path.append(os.path.join(maya_install_path, "bin"))
		sys.path.append(os.path.join(maya_install_path, "Python"))
		sys.path.append(os.path.join(maya_install_path, "Python", maya_version, "DLLs"))
		sys.path.append(os.path.join(maya_install_path, "Python", maya_version, "lib"))
		sys.path.append(os.path.join(maya_install_path, "Python", maya_version, "lib", "lib-tk"))
		sys.path.append(os.path.join(maya_install_path, "Python", maya_version, "lib", "plat-win"))
		sys.path.append(os.path.join(maya_install_path, "Python", maya_version, "lib", "site-packages"))
		sys.path.append(os.path.join(maya_install_path, "devkit", "other", "pymel", "extras", "modules"))
		sys.path.append(os.path.join(maya_install_path, "devkit", "other", "pymel", "extras", "completion"))


	@staticmethod
	def wrapControl(controlName, container):
		'''Embed a Maya Native UI Object.

		Parameters:
			controlName (str): The name of an existing maya control. ie. 'cmdScrollFieldReporter1'
			container (obj): A widget instance in which to wrap the control.

		Example:
			modelPanelName = pm.modelPanel("embeddedModelPanel#", cam='persp')
			wrapControl(modelPanelName, QtWidgets.QtWidget())
		'''
		from PySide2 import QtWidgets
		from shiboken2 import wrapInstance
		from maya.OpenMayaUI import MQtUtil

		layout = QtWidgets.QVBoxLayout(container)
		layout.setContentsMargins(0,0,0,0)
		layoutName = Str.setCase(container.objectName()+'Layout', 'camel') # results in '<objectName>Layout' or 'layout' if container objectName is ''
		layout.setObjectName(layoutName)
		pm.setParent(layoutName)

		from uitk.switchboard import Switchboard
		derivedClass = Switchboard.getDerivedType(container)

		ptr = MQtUtil.findControl(controlName) #get a pointer to the maya api paneLayout.
		control = wrapInstance(int(ptr), derivedClass)
		layout.addWidget(control)

		return control


	@staticmethod
	def mfnMeshGenerator(objects):
		'''Generate mfn mesh from the given list of objects.

		Parameters:
			objects (str)(obj(list): The objects to convert to mfn mesh.

		Return:
			(generator)
		'''
		import maya.OpenMaya as om

		selectionList = om.MSelectionList()
		for mesh in Node.getShapeNode(pm.ls(objects)):
			selectionList.add(mesh)

		for i in range(selectionList.length()):    
			dagPath = om.MDagPath()
			selectionList.getDagPath(i, dagPath)
			# print (dagPath.fullPathName()) #debug
			mfnMesh = om.MFnMesh(dagPath)
			yield mfnMesh


	@staticmethod
	def getArrayType(lst):
		'''Determine if the given element(s) type.
		Samples only the first element.

		Parameters:
			obj (str)(obj)(list): The components(s) to query.

		Return:
			(list) 'str', 'obj'(shape node), 'transform'(as string), 'int'(valid only at sub-object level)

		Example:
		getArrayType('cyl.vtx[0]') #returns: 'transform'
		getArrayType('cylShape.vtx[:]') #returns: 'str'
		'''
		try:
			o = Iter.makeList(lst)[0]
		except IndexError as error:
			# print (f'# Error: {__file__} in getArrayType:\n#\tOperation requires at least one object.\n#\t{error}')
			return ''

		return 'str' if isinstance(o, str) else 'int' if isinstance(o, int) else 'obj'


	@staticmethod
	def convertArrayType(lst, returnType='str', flatten=False):
		'''Convert the given element(s) to <obj>, 'str', or int values.

		Parameters:
			lst (str)(obj)(list): The components(s) to convert.
			returnType (str): The desired returned array element type.
				valid: 'str'(default), 'obj', 'int'(valid only at sub-object level).
			flatten (bool): Flattens the returned list of objects so that each component is it's own element.

		Return:
			(list)(dict) return a dict only with a return type of 'int' and more that one object given.

		Example:
		convertArrayType('obj.vtx[:2]', 'str') #returns: ['objShape.vtx[0:2]']
		convertArrayType('obj.vtx[:2]', 'str', True) #returns: ['objShape.vtx[0]', 'objShape.vtx[1]', 'objShape.vtx[2]']
		convertArrayType('obj.vtx[:2]', 'obj') #returns: [MeshVertex('objShape.vtx[0:2]')]
		convertArrayType('obj.vtx[:2]', 'obj', True) #returns: [MeshVertex('objShape.vtx[0]'), MeshVertex('objShape.vtx[1]'), MeshVertex('objShape.vtx[2]')]
		convertArrayType('obj.vtx[:2]', 'int')) #returns: {nt.Mesh('objShape'): [(0, 2)]}
		convertArrayType('obj.vtx[:2]', 'int', True)) #returns: {nt.Mesh('objShape'): [0, 1, 2]}
		'''
		lst = pm.ls(lst, flatten=flatten)
		if not lst or isinstance(lst[0], int):
			return []

		if returnType=='int':
			result={}
			for c in lst:
				obj = pm.ls(c, objectsOnly=1)[0]
				num = c.split('[')[-1].rstrip(']')

				try:
					if flatten:
						componentNum = int(num)
					else:
						n = [int(n) for n in num.split(':')]
						componentNum = tuple(n) if len(n)>1 else n[0]

					if obj in result: #append to existing object key.
						result[obj].append(componentNum)
					else:
						result[obj] = [componentNum]
				except ValueError as error: #incompatible object type.
					print(f'# Error: {__file__} in convertArrayType\n#\tunable to convert {obj} {num} to int.\n#\t{error}')
					break

			objects = set(pm.ls(lst, objectsOnly=True))
			if len(objects)==1: #flatten the dict values from 'result' and remove any duplicates.
				flattened = Iter.flatten(result.values())
				result = Iter.removeDuplicates(flattened)

		elif returnType=='str':
			result = list(map(str, lst))

		else:
			result = lst

		return result


	@staticmethod
	def getParameterValuesMEL(node, cmd, parameters):
		'''Query a Maya command, and return a key:value pair for each of the given parameters.

		Parameters:
			node (str)(obj)(list): The object to query attributes of.
			parameters (list): The command parameters to query. ie. ['enableTranslationX','translationX']

		Return:
			(dict) {'parameter name':<value>} ie. {'enableTranslationX': [False, False], 'translationX': [-1.0, 1.0]}

		Example: getParameterValuesMEL(obj, 'transformLimits', ['enableTranslationX','translationX'])
		'''
		cmd = getattr(pm, cmd)
		node = pm.ls(node)[0]

		result={}
		for p in parameters:
			values = cmd(node, **{'q':True, p:True}) #query the parameter to get it's value.

			result[p] = values

		return result


	@staticmethod
	def setParameterValuesMEL(node, cmd, parameters):
		'''Set parameters using a maya command.

		Parameters:
			node (str)(obj)(list): The object to query attributes of.
			parameters (dict): The command's parameters and their desired values. ie. {'enableTranslationX': [False, False], 'translationX': [-1.0, 1.0]}

		Example: setParameterValuesMEL(obj, 'transformLimits', {'enableTranslationX': [False, False], 'translationX': [-1.0, 1.0]})
		'''
		cmd = getattr(pm, cmd)
		node = pm.ls(node)[0]

		for p, v in parameters.items():
			cmd(node, **{p:v})


	@staticmethod
	def getSelectedChannels():
		'''Get any attributes (channels) that are selected in the channel box.

		Return:
			(str) list of any selected attributes as strings. (ie. ['tx', ry', 'sz'])
		'''
		channelBox = pm.mel.eval('global string $gChannelBoxName; $temp=$gChannelBoxName;') #fetch maya's main channelbox
		attrs = pm.channelBox(channelBox, q=True, sma=True)

		if attrs is None:
			attrs=[]
		return attrs


	@staticmethod
	def getPanel(*args, **kwargs):
		'''Returns panel and panel configuration information.
		A fix for the broken pymel command of the same name.

		Parameters:
			[allConfigs=boolean], [allPanels=boolean], [allScriptedTypes=boolean], [allTypes=boolean], [configWithLabel=string], [containing=string], [invisiblePanels=boolean], [scriptType=string], [type=string], [typeOf=string], [underPointer=boolean], [visiblePanels=boolean], [withFocus=boolean], [withLabel=string])

		Return:
			(str) An array of panel names.
		'''
		from maya.cmds import getPanel #pymel getPanel is broken in ver: 2022.

		result = getPanel(*args, **kwargs)

		return result


	@staticmethod
	def mainProgressBar(size, name="progressBar#", stepAmount=1):
		'''#add esc key pressed return False

		Parameters:
			size (int): total amount
			name (str): name of progress bar created
			stepAmount(int): increment amount

		example use-case:
		mainProgressBar (len(edges), progressCount)
			pm.progressBar ("progressBar_", edit=1, step=1)
			if pm.progressBar ("progressBar_", query=1, isCancelled=1):
				break
		pm.progressBar ("progressBar_", edit=1, endProgress=1)

		to use main progressBar: name=string $gMainProgressBar
		'''
		status = 'processing: {} items ..'.format(size)

		edit=False
		if pm.progressBar(name, exists=1):
			edit=True

		pm.progressBar(name, edit=edit,
						beginProgress=1,
						isInterruptable=True,
						status=status,
						maxValue=size,
						step=stepAmount)


	@staticmethod
	def viewportMessage(message='', statusMessage='', assistMessage='', position='topCenter'):
		'''
		Parameters:
			message (str): The message to be displayed, (accepts html formatting). General message, inherited by -amg/assistMessage and -smg/statusMessage.
			statusMessage (str): The status info message to be displayed (accepts html formatting).
			assistMessage (str): The user assistance message to be displayed, (accepts html formatting).
			position (str): position on screen. possible values are: topCenter","topRight","midLeft","midCenter","midCenterTop","midCenterBot","midRight","botLeft","botCenter","botRight"

		ex. viewportMessage("shutting down:<hl>"+str(timer)+"</hl>")
		'''
		fontSize=10
		fade=1
		fadeInTime=0
		fadeStayTime=1000
		fadeOutTime=500
		alpha=75

		if message:
			pm.inViewMessage(message=message, position=position, fontSize=fontSize, fade=fade, fadeInTime=fadeInTime, fadeStayTime=fadeStayTime, fadeOutTime=fadeOutTime, alpha=alpha) #1000ms = 1 sec
		elif statusMessage:
			pm.inViewMessage(statusMessage=statusMessage, position=position, fontSize=fontSize, fade=fade, fadeInTime=fadeInTime, fadeStayTime=fadeStayTime, fadeOutTime=fadeOutTime, alpha=alpha) #1000ms = 1 sec
		elif assistMessage:
			pm.inViewMessage(assistMessage=assistMessage, position=position, fontSize=fontSize, fade=fade, fadeInTime=fadeInTime, fadeStayTime=fadeStayTime, fadeOutTime=fadeOutTime, alpha=alpha) #1000ms = 1 sec


	@staticmethod
	def outputText(text, window_title):
		'''output text
		'''
		#window_title = pm.mel.eval(python("window_title"))
		window = str(pm.window(	widthHeight=(300, 300), 
								topLeftCorner=(65,265),
								maximizeButton=False,
								resizeToFitChildren=True,
								toolbox=True,
								title=window_title))
		scrollLayout = str(pm.scrollLayout(verticalScrollBarThickness=16, 
										horizontalScrollBarThickness=16))
		pm.columnLayout(adjustableColumn=True)
		text_field = str(pm.text(label=text, align='left'))
		print(text_field)
		pm.setParent('..')
		pm.showWindow(window)
		return

	# #output textfield parsed by ';'
	# def outputTextField2(text):
	# 	window = str(pm.window(	widthHeight=(250, 650), 
	# 							topLeftCorner=(50,275),
	# 							maximizeButton=False,
	# 							resizeToFitChildren=False,
	# 							toolbox=True,
	# 							title=""))
	# 	scrollLayout = str(pm.scrollLayout(verticalScrollBarThickness=16, 
	# 									horizontalScrollBarThickness=16))
	# 	pm.columnLayout(adjustableColumn=True)
	# 	print(text)
	# 	#for item in array:
	# 	text_field = str(pm.textField(height=20,
	# 										width=250, 
	# 										editable=False,
	# 										insertText=str(text)))
	# 	pm.setParent('..')
	# 	pm.showWindow(window)
	# 	return


	@staticmethod
	def outputscrollField(text, window_title, width, height):
		'''Create an output scroll layout.
		'''
		window_width  = width  * 300
		window_height = height * 600
		scroll_width  = width  * 294
		scroll_height = height * 590
		window = str(pm.window(	widthHeight=(window_width, window_height),
								topLeftCorner=(45, 0),
								maximizeButton=False,
								sizeable=False,
								title=window_title
								))
		scrollLayout = str(pm.scrollLayout(verticalScrollBarThickness=16, 
										horizontalScrollBarThickness=16))
		pm.columnLayout(adjustableColumn=True)
		scroll_field = str(pm.scrollField(text=(text),
										width=scroll_width,
										height=scroll_height,))
		print(window)
		pm.setParent('..')
		pm.showWindow(window)
		return scroll_field


	@staticmethod
	def outputTextField(array, window_title):
		'''Create an output text field.
		'''
		window = str(pm.window(	widthHeight=(250, 650), 
								topLeftCorner=(65,275),
								maximizeButton=False,
								resizeToFitChildren=False,
								toolbox=True,
								title=window_title))
		scrollLayout = str(pm.scrollLayout(verticalScrollBarThickness=16, 
										horizontalScrollBarThickness=16))
		pm.columnLayout(adjustableColumn=True)
		for item in array:
			text_field = str(pm.textField(height=20,
											width=500, 
											editable=False,
											insertText=str(item)))
		pm.setParent('..')
		pm.showWindow(window)
		return text_field

# --------------------------------------------------------------------------------------------









# --------------------------------------------------------------------------------------------

def __getattr__(attr:str):
	"""Searches for an attribute in this module's classes and returns it.

	Parameters:
		attr (str): The name of the attribute to search for.
	
	Return:
		(obj) The found attribute.

	:Raises:
		AttributeError: If the given attribute is not found in any of the classes in the module.
	"""
	try:
		return getattr(Core, attr)

	except AttributeError as error:
		raise AttributeError(f"Module '{__name__}' has no attribute '{attr}'")

# --------------------------------------------------------------------------------------------

if __name__=='__main__':
	pass

# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------



# --------------------------------------------------------------------------------------------
# deprecated:
# --------------------------------------------------------------------------------------------